@@ -3,11 +3,33 @@ import enum
 import json
 from collections.abc import Mapping
 from functools import reduce
+from typing import Dict
 
+from typedpy.fields import _ListStruct
 from typedpy.structures import TypedField, Structure, _get_all_fields_by_name
-from typedpy.fields import Field, Number, String, StructureReference, \
-    Array, Map, ClassReference, Enum, MultiFieldWrapper, Boolean, Tuple, Set, Anything, AnyOf, AllOf, \
-    OneOf, NotField, SerializableField, SizedCollection, wrap_val, Function
+from typedpy.fields import (
+    Field,
+    Number,
+    String,
+    StructureReference,
+    Array,
+    Map,
+    ClassReference,
+    Enum,
+    MultiFieldWrapper,
+    Boolean,
+    Tuple,
+    Set,
+    Anything,
+    AnyOf,
+    AllOf,
+    OneOf,
+    NotField,
+    SerializableField,
+    SizedCollection,
+    wrap_val,
+    Function, _DictStruct,
+)
 
 
 def deserialize_list_like(field, content_type, value, name):
@@ -25,7 +47,9 @@ def deserialize_list_like(field, content_type, value, name):
             try:
                 list_item = deserialize_single_field(items, v, item_name)
             except (ValueError, TypeError) as e:
-                prefix = "" if str(e).startswith(item_name) else "{}: ".format(item_name)
+                prefix = (
+                    "" if str(e).startswith(item_name) else "{}: ".format(item_name)
+                )
                 raise ValueError("{}{}".format(prefix, str(e)))
             values.append(list_item)
     else:
@@ -60,34 +84,48 @@ def deserialize_multifield_wrapper(field, source_val, name, keep_undefined=True)
     found_previous_match = False
     for field_option in field.get_fields():
         try:
-            deserialized = deserialize_single_field(field_option, source_val, name, keep_undefined=keep_undefined)
+            deserialized = deserialize_single_field(
+                field_option, source_val, name, keep_undefined=keep_undefined
+            )
             if isinstance(field, AnyOf):
                 return deserialized
             elif isinstance(field, NotField):
-                raise ValueError("could not deserialize {}: value {} matches field {}, but must not match it".format(
-                    name, wrap_val(source_val), field))
+                raise ValueError(
+                    "could not deserialize {}: value {} matches field {}, but must not match it".format(
+                        name, wrap_val(source_val), field
+                    )
+                )
             elif isinstance(field, OneOf) and found_previous_match:
-                raise ValueError("could not deserialize {}: value {} matches more than one match".format(
-                    name, wrap_val(source_val), field))
+                raise ValueError(
+                    "could not deserialize {}: value {} matches more than one match".format(
+                        name, wrap_val(source_val), field
+                    )
+                )
             found_previous_match = True
         except Exception as e:
             if isinstance(field, AllOf):
-                raise ValueError("could not deserialize {}: value {} did not match {}. reason: {}".format(
-                    name, wrap_val(source_val), field_option, str(e)))
+                raise ValueError(
+                    "could not deserialize {}: value {} did not match {}. reason: {}".format(
+                        name, wrap_val(source_val), field_option, str(e)
+                    )
+                )
     return deserialized
 
 
 def deserialize_map(map_field, source_val, name):
     if not isinstance(source_val, dict):
-        raise TypeError("{}: expected a dict. Got {}".format(name, wrap_val(source_val)))
+        raise TypeError(
+            "{}: expected a dict. Got {}".format(name, wrap_val(source_val))
+        )
     if map_field.items:
         key_field, value_field = map_field.items
     else:
         key_field, value_field = None, None
     res = {}
     for key, val in source_val.items():
-        res[deserialize_single_field(key_field, key, name)] = \
-            deserialize_single_field(value_field, val, name)
+        res[deserialize_single_field(key_field, key, name)] = deserialize_single_field(
+            value_field, val, name
+        )
     return res
 
 
@@ -95,9 +133,11 @@ def deserialize_single_field(field, source_val, name, keep_undefined=True):
     if isinstance(field, (Number, String, Enum, Boolean)):
         field._validate(source_val)
         value = source_val
-    elif isinstance(field, TypedField) and \
-            getattr(field, '_ty', '') in {str, int, float} and \
-            isinstance(source_val, getattr(field, '_ty', '')):
+    elif (
+            isinstance(field, TypedField)
+            and getattr(field, "_ty", "") in {str, int, float}
+            and isinstance(source_val, getattr(field, "_ty", ""))
+    ):
         value = source_val
     elif isinstance(field, Array):
         value = deserialize_array(field, source_val, name)
@@ -106,11 +146,17 @@ def deserialize_single_field(field, source_val, name, keep_undefined=True):
     elif isinstance(field, Set):
         value = deserialize_set(field, source_val, name)
     elif isinstance(field, MultiFieldWrapper):
-        value = deserialize_multifield_wrapper(field, source_val, name, keep_undefined=keep_undefined)
+        value = deserialize_multifield_wrapper(
+            field, source_val, name, keep_undefined=keep_undefined
+        )
     elif isinstance(field, ClassReference):
-        value = deserialize_structure_internal(getattr(field, '_ty'), source_val, name, keep_undefined=keep_undefined)
+        value = deserialize_structure_internal(
+            getattr(field, "_ty"), source_val, name, keep_undefined=keep_undefined
+        )
     elif isinstance(field, StructureReference):
-        value = deserialize_structure_reference(getattr(field, '_newclass'), source_val, keep_undefined=keep_undefined)
+        value = deserialize_structure_reference(
+            getattr(field, "_newclass"), source_val, keep_undefined=keep_undefined
+        )
     elif isinstance(field, Map):
         value = deserialize_map(field, source_val, name)
     elif isinstance(field, SerializableField):
@@ -118,15 +164,29 @@ def deserialize_single_field(field, source_val, name, keep_undefined=True):
     elif isinstance(field, Anything) or field is None:
         value = source_val
     else:
-        raise NotImplementedError("cannot deserialize field '{}' of type {} using value {}".
-                                  format(name, field.__class__.__name__, wrap_val(source_val)))
+        raise NotImplementedError(
+            "cannot deserialize field '{}' of type {} using value {}".format(
+                name, field.__class__.__name__, wrap_val(source_val)
+            )
+        )
     return value
 
 
 def deserialize_structure_reference(cls, the_dict: dict, keep_undefined):
-    field_by_name = dict([(k, v) for k, v in cls.__dict__.items()
-                          if isinstance(v, Field) and k in the_dict])
-    kwargs = dict([(k, v) for k, v in the_dict.items() if k not in field_by_name and keep_undefined])
+    field_by_name = dict(
+        [
+            (k, v)
+            for k, v in cls.__dict__.items()
+            if isinstance(v, Field) and k in the_dict
+        ]
+    )
+    kwargs = dict(
+        [
+            (k, v)
+            for k, v in the_dict.items()
+            if k not in field_by_name and keep_undefined
+        ]
+    )
 
     for name, field in field_by_name.items():
         kwargs[name] = deserialize_single_field(field, the_dict[name], name)
@@ -146,33 +206,36 @@ class FunctionCall(Structure):
         args(Array[String]): optional
             the keys of the arguments to be used as positional arguments for the function call
     """
+
     func = Function
     args = Array[String]
-    _required = ['func']
+    _required = ["func"]
 
 
-def deserialize_structure_internal(cls, the_dict, name=None, *, mapper=None, keep_undefined=True):
+def deserialize_structure_internal(
+        cls, the_dict, name=None, *, mapper=None, keep_undefined=True
+):
     """
-        Deserialize a dict to a Structure instance, Jackson style.
-        Note the top level must be a python dict - which implies that a JSON of
-        simply a number, or string, or array, is unsupported.
-        `See working examples in test. <https://github.com/loyada/typedpy/tree/master/tests/test_deserialization.py>`_
+    Deserialize a dict to a Structure instance, Jackson style.
+    Note the top level must be a python dict - which implies that a JSON of
+    simply a number, or string, or array, is unsupported.
+    `See working examples in test. <https://github.com/loyada/typedpy/tree/master/tests/test_deserialization.py>`_
 
-        Arguments:
-            cls(type):
-                The target class
-            the_dict(dict):
-                the source dictionary
-            mapper(dict): optional
-                a dict of attribute name of attribute to key in the input
-            name(str): optional
-                name of the structure, used only internally, when there is a
-                class reference field. Users are not supposed to use this argument.
-            keep_undefined(bool): optional
-                should it create attributes for keys that don't appear in the class? default is False.
+    Arguments:
+        cls(type):
+            The target class
+        the_dict(dict):
+            the source dictionary
+        mapper(dict): optional
+            a dict of attribute name of attribute to key in the input
+        name(str): optional
+            name of the structure, used only internally, when there is a
+            class reference field. Users are not supposed to use this argument.
+        keep_undefined(bool): optional
+            should it create attributes for keys that don't appear in the class? default is False.
 
-        Returns:
-            an instance of the provided :class:`Structure` deserialized
+    Returns:
+        an instance of the provided :class:`Structure` deserialized
     """
     if mapper is None:
         mapper = {}
@@ -183,14 +246,24 @@ def deserialize_structure_internal(cls, the_dict, name=None, *, mapper=None, kee
     if not isinstance(the_dict, dict):
         props = cls.__dict__
         fields = list(field_by_name.keys())
-        required = props.get('_required', fields)
-        additional_props = props.get('_additionalProperties', True)
+        required = props.get("_required", fields)
+        additional_props = props.get("_additionalProperties", True)
         if len(fields) == 1 and required == fields and additional_props is False:
             field_name = fields[0]
-            return cls(deserialize_single_field(getattr(cls, field_name), the_dict, field_name))
-        raise TypeError("{}: Expected a dictionary; Got {}".format(name, wrap_val(the_dict)))
+            return cls(
+                deserialize_single_field(getattr(cls, field_name), the_dict, field_name)
+            )
+        raise TypeError(
+            "{}: Expected a dictionary; Got {}".format(name, wrap_val(the_dict))
+        )
 
-    kwargs = dict([(k, v) for k, v in the_dict.items() if k not in field_by_name and keep_undefined])
+    kwargs = dict(
+        [
+            (k, v)
+            for k, v in the_dict.items()
+            if k not in field_by_name and keep_undefined
+        ]
+    )
     for key, field in field_by_name.items():
         process = False
         if key in the_dict and key not in mapper:
@@ -200,41 +273,45 @@ def deserialize_structure_internal(cls, the_dict, name=None, *, mapper=None, kee
             processed_input = get_processed_input(key, mapper, the_dict)
             process = True
         if process:
-            kwargs[key] = deserialize_single_field(field, processed_input, key, keep_undefined=keep_undefined)
+            kwargs[key] = deserialize_single_field(
+                field, processed_input, key, keep_undefined=keep_undefined
+            )
 
     return cls(**kwargs)
 
 
 def deserialize_structure(cls, the_dict, *, mapper=None, keep_undefined=True):
     """
-        Deserialize a dict to a Structure instance, Jackson style.
-        Note the top level must be a python dict - which implies that a JSON of
-        simply a number, or string, or array, is unsupported.
-        `See working examples in test. <https://github.com/loyada/typedpy/tree/master/tests/test_deserialization.py>`_
+    Deserialize a dict to a Structure instance, Jackson style.
+    Note the top level must be a python dict - which implies that a JSON of
+    simply a number, or string, or array, is unsupported.
+    `See working examples in test. <https://github.com/loyada/typedpy/tree/master/tests/test_deserialization.py>`_
 
-        Arguments:
-            cls(type):
-                The target class
-            the_dict(dict):
-                the source dictionary
-            mapper(dict): optional
-                the key is the target attribute name. The value can either be a path of the value in the source dict
-                using dot notation, for example: "aaa.bbb", or a :class:`FunctionCall`. In the latter case,
-                the function is the used to preprocess the input prior to deserialization/validation.
-                The args attribute in the function call is optional. If non provided, the input to the function is
-                the value with the same key. Otherwise it is the keys of the values in the input that are injected
-                to the provided function. See working examples in the tests link above.
-            keep_undefined(bool): optional
-                should it create attributes for keys that don't appear in the class? default is True.
+    Arguments:
+        cls(type):
+            The target class
+        the_dict(dict):
+            the source dictionary
+        mapper(dict): optional
+            the key is the target attribute name. The value can either be a path of the value in the source dict
+            using dot notation, for example: "aaa.bbb", or a :class:`FunctionCall`. In the latter case,
+            the function is the used to preprocess the input prior to deserialization/validation.
+            The args attribute in the function call is optional. If non provided, the input to the function is
+            the value with the same key. Otherwise it is the keys of the values in the input that are injected
+            to the provided function. See working examples in the tests link above.
+        keep_undefined(bool): optional
+            should it create attributes for keys that don't appear in the class? default is True.
 
-        Returns:
-            an instance of the provided :class:`Structure` deserialized
+    Returns:
+        an instance of the provided :class:`Structure` deserialized
     """
-    return deserialize_structure_internal(cls, the_dict, mapper=mapper, keep_undefined=keep_undefined)
+    return deserialize_structure_internal(
+        cls, the_dict, mapper=mapper, keep_undefined=keep_undefined
+    )
 
 
 def _deep_get(dictionary, deep_key):
-    keys = deep_key.split('.')
+    keys = deep_key.split(".")
     return reduce(lambda d, key: d.get(key) if d else None, keys, dictionary)
 
 
@@ -242,19 +319,29 @@ def get_processed_input(key, mapper, the_dict):
     if key in mapper:
         key_mapper = mapper[key]
         if isinstance(key_mapper, (FunctionCall,)):
-            args = [_deep_get(the_dict, k) for k in key_mapper.args] if key_mapper.args else [the_dict.get(key)]
+            args = (
+                [_deep_get(the_dict, k) for k in key_mapper.args]
+                if key_mapper.args
+                else [the_dict.get(key)]
+            )
             processed_input = key_mapper.func(*args)
         elif isinstance(key_mapper, (str,)):
             processed_input = _deep_get(the_dict, key_mapper)
         else:
-            raise TypeError("mapper value must be a key in the input or a FunctionCal. Got {}".format(wrap_val(key_mapper)))
+            raise TypeError(
+                "mapper value must be a key in the input or a FunctionCal. Got {}".format(
+                    wrap_val(key_mapper)
+                )
+            )
     else:
         processed_input = the_dict[key]
     return processed_input
 
 
 def serialize_val(field_definition, name, val):
-    if isinstance(field_definition, SerializableField) and isinstance(field_definition, Field):
+    if isinstance(field_definition, SerializableField) and isinstance(
+            field_definition, Field
+    ):
         return field_definition.serialize(val)
     if isinstance(val, (int, str, bool, float)) or val is None:
         return val
@@ -264,13 +351,25 @@ def serialize_val(field_definition, name, val):
         if isinstance(field_definition, Map):
             if not isinstance(val, Mapping):
                 raise TypeError("{} Expected a Mapping", name)
-            if isinstance(field_definition.items, list) and len(field_definition.items) == 2:
+            if (
+                    isinstance(field_definition.items, list)
+                    and len(field_definition.items) == 2
+            ):
                 key_type, value_type = field_definition.items
-                return {serialize_val(key_type, name, k): serialize_val(value_type, name, v) for (k, v) in val.items()}
+                return {
+                    serialize_val(key_type, name, k): serialize_val(value_type, name, v)
+                    for (k, v) in val.items()
+                }
             else:
-                return {serialize_val(Anything, name, k) : serialize_val(Anything, name, v) for (k, v) in val.items()}
+                return {
+                    serialize_val(Anything, name, k): serialize_val(Anything, name, v)
+                    for (k, v) in val.items()
+                }
         if isinstance(field_definition.items, list):
-            return [serialize_val(field_definition.items[ind], name, v) for ind, v in enumerate(val)]
+            return [
+                serialize_val(field_definition.items[ind], name, v)
+                for ind, v in enumerate(val)
+            ]
         elif isinstance(field_definition.items, Field):
             return [serialize_val(field_definition.items, name, i) for i in val]
         else:
@@ -327,9 +426,13 @@ def _get_mapped_value(mapper, key, items):
     if key in mapper:
         key_mapper = mapper[key]
         if isinstance(key_mapper, (FunctionCall,)):
-            args = [items.get(k) for k in key_mapper.args] if key_mapper.args else [items.get(key)]
+            args = (
+                [items.get(k) for k in key_mapper.args]
+                if key_mapper.args
+                else [items.get(key)]
+            )
             return key_mapper.func(*args)
-        elif not isinstance(key_mapper, (FunctionCall,str)):
+        elif not isinstance(key_mapper, (FunctionCall, str)):
             raise TypeError("mapper must have a FunctionCall or a string")
 
     return None
@@ -340,37 +443,55 @@ def serialize_internal(structure, mapper=None, compact=False):
         mapper = {}
     field_by_name = _get_all_fields_by_name(structure.__class__)
 
-    items = structure.items() if isinstance(structure, dict) \
-        else [(k, v) for (k, v) in structure.__dict__.items() if k != '_instantiated']
+    items = (
+        structure.items()
+        if isinstance(structure, dict)
+        else [(k, v) for (k, v) in structure.__dict__.items() if k != "_instantiated"]
+    )
     props = structure.__class__.__dict__
     fields = list(field_by_name.keys())
-    required = props.get('_required', fields)
-    additional_props = props.get('_additionalProperties', True)
-    if len(fields) == 1 and required == fields \
-            and additional_props is False and compact:
+    additional_props = props.get("_additionalProperties", True)
+    if (
+            len(fields) == 1
+            and props.get("_required", fields) == fields
+            and additional_props is False
+            and compact
+    ):
         key = fields[0]
-        result = serialize_val(field_by_name.get(key, None), key, getattr(structure, key))
+        result = serialize_val(
+            field_by_name.get(key, None), key, getattr(structure, key)
+        )
     else:
         result = {}
         items_map = dict(items)
         for key, val in items:
             if val is None:
                 continue
-            mapped_key = mapper[key] if key in mapper and isinstance(mapper[key], (str,)) else key
+            mapped_key = (
+                mapper[key]
+                if key in mapper and isinstance(mapper[key], (str,))
+                else key
+            )
             mapped_value = _get_mapped_value(mapper, key, items_map)
-            the_field_definition = Anything if mapped_value else field_by_name.get(key, None)
-            result[mapped_key] = serialize_val(the_field_definition, key, mapped_value or val)
+            the_field_definition = (
+                Anything if mapped_value else field_by_name.get(key, None)
+            )
+            result[mapped_key] = serialize_val(
+                the_field_definition, key, mapped_value or val
+            )
     return result
 
 
-def serialize(structure, *, mapper=None, compact=False):
+def serialize(value, *, mapper: Dict = None, compact=False):
     """
     Serialize an instance of :class:`Structure` to a JSON-like dict.
     `See working examples in test. <https://github.com/loyada/typedpy/tree/master/tests/test_serialization.py>`_
 
     Arguments:
-        structure(:class:`Structure`):
-            the structure instance to be serialized to JSON
+        value(:class:`Structure` or a field value with an obvious serialization):
+            The value to be serialized - a structure instance, or a field value for which typedpy can deduce the
+             serialization.
+            In the general case, if you just need to serialize a field value, it's better to use serialize_field().
 
         mapper(dict): optional
              a dictionary where the key is the name of the attribute in the structure, and the value is name of the
@@ -392,8 +513,14 @@ def serialize(structure, *, mapper=None, compact=False):
         mapper = {}
     if not isinstance(mapper, (collections.abc.Mapping,)):
         raise TypeError("Mapper must be a mapping")
-    if not isinstance(structure, Structure):
-        if structure is None:
-            return None
-        raise TypeError("serialize: must get a Structure. Got: {}".format(structure))
-    return serialize_internal(structure, mapper=mapper, compact=compact)
+    if not isinstance(value, (Structure, StructureReference)):
+        if value is None or isinstance(value, (int, str, bool, float)):
+            return value
+        if isinstance(value, (_ListStruct, _DictStruct)):
+            field_definition = value._field_definition
+            return serialize_val(field_definition, field_definition._name, value)
+        if isinstance(value, (enum.Enum,)):
+            return value.name
+        raise TypeError("serialize: Not a Structure or Field that with an obvious serialization. Got: {}."
+                        " Maybe try serialize_field() instead?".format(value))
+    return serialize_internal(value, mapper=mapper, compact=compact)
