@@ -3,50 +3,51 @@ from typedpy.fields import StructureClass, Map, String, OneOf
 from typedpy.serialization import FunctionCall, deserialize_structure, serialize
 from typedpy.structures import _get_all_fields_by_name
 
+
 class Deserializer(Structure):
     """
-        A high level API for a deserializer: from a dict or anything else that could be sent as a JSON, to
-        a :class:`Structure`.
-        The advantage of this over the lower level function is that it is more explicit and self-validating.
-        In other words, it prevents the user from creating an invalid mapper.
+    A high level API for a deserializer: from a dict or anything else that could be sent as a JSON, to
+    a :class:`Structure`.
+    The advantage of this over the lower level function is that it is more explicit and self-validating.
+    In other words, it prevents the user from creating an invalid mapper.
 
-        Arguments:
-            target_class(:class:`StructureClass`):
-                A class extending the abstract :class:`Structure` that this deserializer is build for.
-                Example:
+    Arguments:
+        target_class(:class:`StructureClass`):
+            A class extending the abstract :class:`Structure` that this deserializer is build for.
+            Example:
 
-                .. code-block:: python
+            .. code-block:: python
 
-                    class Foo(Structure):
-                        id = Integer
-                        name = String
+                class Foo(Structure):
+                    id = Integer
+                    name = String
 
-                   Deserializer(target_class=Foo)
+               Deserializer(target_class=Foo)
 
 
-            mapper(dict): optional
-                The key is the target attribute name. The value can either be a path of the value in the source dict
-                using dot notation, for example: "aaa.bbb", or a :class:`FunctionCall`. In the latter case,
-                the function is the used to preprocess the input prior to deserialization/validation.
-                The args attribute in the function call is optional. If non provided, the input to the function is
-                the value with the same key. Otherwise it is the keys of the values in the input that are injected
-                to the provided function. See working examples in the tests link above.
-                This class will ensure that the mapper is a valid one for its target_class.
-                Example:
+        mapper(dict): optional
+            The key is the target attribute name. The value can either be a path of the value in the source dict
+            using dot notation, for example: "aaa.bbb", or a :class:`FunctionCall`. In the latter case,
+            the function is the used to preprocess the input prior to deserialization/validation.
+            The args attribute in the function call is optional. If non provided, the input to the function is
+            the value with the same key. Otherwise it is the keys of the values in the input that are injected
+            to the provided function. See working examples in the tests link above.
+            This class will ensure that the mapper is a valid one for its target_class.
+            Example:
 
-                .. code-block:: python
+            .. code-block:: python
 
-                     class Foo(Structure):
-                        m = Map
-                        s = String
-                        i = Integer
+                 class Foo(Structure):
+                    m = Map
+                    s = String
+                    i = Integer
 
-                    mapper = {
-                        "m": "a.b",
-                        "s": FunctionCall(func=lambda x: f'the string is {x}', args=['name.first']),
-                        'i': FunctionCall(func=operator.add, args=['i', 'j'])
-                    }
-                    Deserializer(target_class=Foo, mapper = mapper).deserializer(the_input_dict)
+                mapper = {
+                    "m": "a.b",
+                    "s": FunctionCall(func=lambda x: f'the string is {x}', args=['name.first']),
+                    'i': FunctionCall(func=operator.add, args=['i', 'j'])
+                }
+                Deserializer(target_class=Foo, mapper = mapper).deserializer(the_input_dict)
 
 
     """
@@ -54,7 +55,7 @@ class Deserializer(Structure):
     target_class = StructureClass
     mapper = Map[String, OneOf[String, FunctionCall]]
 
-    _required = ['target_class']
+    _required = ["target_class"]
 
     def __validate__(self):
         valid_keys = set(_get_all_fields_by_name(self.target_class).keys())
@@ -63,14 +64,14 @@ class Deserializer(Structure):
                 if key not in valid_keys:
                     raise ValueError(
                         "Invalid key in mapper for class {}: {}. Keys must be one of the class fields.".format(
-                            self.target_class.__name__, key))
+                            self.target_class.__name__, key
+                        )
+                    )
 
     def deserialize(self, input, keep_undefined=True):
-        return deserialize_structure(self.target_class,
-                                     input,
-                                     mapper=self.mapper,
-                                     keep_undefined=keep_undefined
-                                     )
+        return deserialize_structure(
+            self.target_class, input, mapper=self.mapper, keep_undefined=keep_undefined
+        )
 
 
 class Serializer(Structure):
@@ -120,10 +121,11 @@ class Serializer(Structure):
 
 
     """
+
     source = Structure
     mapper = Map[String, OneOf[String, FunctionCall]]
 
-    _required = ['source']
+    _required = ["source"]
 
     def __validate__(self):
         source_class = self.source.__class__
@@ -131,17 +133,21 @@ class Serializer(Structure):
         if self.mapper:
             for key in self.mapper:
                 if key not in valid_keys:
-                    raise ValueError("Invalid key in mapper for class {}: {}. Keys must be one of the class fields.".format(
-                        source_class.__name__, key))
+                    raise ValueError(
+                        "Invalid key in mapper for class {}: {}. Keys must be one of the class fields.".format(
+                            source_class.__name__, key
+                        )
+                    )
                 if isinstance(self.mapper[key], (FunctionCall,)):
                     args = self.mapper[key].args
                     if isinstance(args, (list,)):
                         for arg in args:
                             if arg not in valid_keys:
                                 raise ValueError(
-                                    "Mapper[{}] has a function call with an invalid argument: {}".format(key, arg))
+                                    "Mapper[{}] has a function call with an invalid argument: {}".format(
+                                        key, arg
+                                    )
+                                )
 
     def serialize(self, compact=True):
-        return serialize(self.source,
-                         mapper=self.mapper,
-                         compact=compact)
+        return serialize(self.source, mapper=self.mapper, compact=compact)
